//--------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//--------------------------------------------------------------------------
//
//	basicdefs.h --- useful constants and macros of miscellaneous nature ---
//
//	DESCRIPTION:
//		This include file defines several useful macros
//
//	EXTERNAL DEPENDENCIES:
//		function			package				comments
//		----------------------------------------------------------------------
//
//	CHANGE HISTORY:
//		UL, 1997-08-26: 1.0.0: - first version
//		UL, 1998-06-05: 1.1.0: - more macros added
//
//	AUTHOR:
//		U. Lammers (UL), Wed Aug 27 11:48:12 MET DST 1997
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/dsslib/src/build/basicdefs.h,v 1.2 2013/11/11 17:15:41 aibarra Exp $
//

#ifndef BASIC_H
#define BASIC_H

#ifdef __cplusplus
	#include <cassert>

	#define YYTEXT		(YYText())
#else
	#include <assert.h>

	#define YYTEXT		(yytext)
#endif

#define BOOL			int
#if defined(TRUE)
	#undef TRUE
#endif
#define TRUE			(1)
#if defined(FALSE)
	#undef FALSE
#endif
#define FALSE			(0)

//
//	-------------------( useful constants )------------------------------
//

#ifdef M_PI
	#undef M_PI
#endif
const double M_PI			= 3.14159265358979323846;	// Pi
const double M_HALF_PI		= .5*M_PI;					// Pi/2
const double M_TWO_PI		= 2.*M_PI;					// 2*Pi
const double M_SQRT_TWO_PI	= 2.5066282746310005024;	// sqrt(2.*Pi)
const double M_2SQ2LN2		= 2.3548200450309493820;	// 2.*sqrt(2.*ln(2))
#ifdef M_E
	#undef M_E
#endif
const double M_E			= 2.7182818284590452354;	// E
const double EPSILON		= 1.e-12;
const double DEG_TO_RAD		= M_PI/180.;
const double RAD_TO_DEG		= 180./M_PI;


//
//	------------------( conversion functions )-----------------------------
//

#define DBL(a)			(static_cast<double>(a))
#define INT(a)			(static_cast<int>(a))
#define UNT(a)			(static_cast<unsigned>(a))
#define LNG(a)			(static_cast<long>(a))
#define ULNG(a)			(static_cast<unsigned long>(a))
#define UINT(a)			(static_cast<uint32_t>(a))

//
//	---------------------( useful macros )--------------------------------
 
#define TOSTR(a)			#a
#define CSTR(a)				TOSTR(a)

#define YCAT(a, b)			a ## b
#define CAT(a, b)			YCAT(a, b)
#define CCAT(a, b, c)		CAT(CAT(a, b), c)

#define NUM_OF(x)			(sizeof(x)/sizeof(x[0]))
#define STREQ(a, b)			(strcmp((a), (b)) == 0)
#define STRIEQ(a, b)		(stricmp((a), (b)) == 0)
#define STRNCPY(a, b)		(strncpy(a, b, sizeof(a)))
#define TMALLOC(a)			((a *)malloc(sizeof(a)))
#define TSMALLOC(a, n)		((a *)malloc((n) * sizeof(a)))
#define TGETMEM(var, type)	{ var = TMALLOC(type); assert(var); }

#ifdef MIN
	#undef MIN
#endif
#define MIN(a, b)			((a) <= (b) ? (a) : (b))
#ifdef MAX
	#undef MAX
#endif
#define MAX(a, b)			((a) >= (b) ? (a) : (b))

#define MINMAX(a, b, c)		(MIN((c), MAX((a), (b))))

#define PERCENTAGE(a, b)	((b) ? (100. * (double)(a)/(double)(b)) : 100.)

#endif
